﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HeuristicLab.BioBoost.Operators.Mutation;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.BioBoost.Operators.Moves {

  [StorableClass]
  public class MoveGeneratorAdapter : SingleSuccessorOperator, IMoveGenerator {

    public IConstrainedValueParameter<IManipulator> ManipulatorParameter { get { return (IConstrainedValueParameter<IManipulator>) Parameters["Manipulator"]; } }
    public IManipulator Manipulator { get { return ManipulatorParameter.Value; } }

    public IValueLookupParameter<IntValue> SampleSizeParameter { get { return (IValueLookupParameter<IntValue>) Parameters["SampleSize"]; } }
    public int SampleSize { get { return SampleSizeParameter.ActualValue.Value; } }

    #region construction & cloning
    [StorableConstructor]
    public MoveGeneratorAdapter(bool isDeserializing) : base(isDeserializing) { }

    public MoveGeneratorAdapter(MoveGeneratorAdapter orig, Cloner cloner) : base(orig, cloner) { }

    public MoveGeneratorAdapter() {
      Parameters.Add(new ValueLookupParameter<IntValue>("SampleSize", "The number of moves to generate."));
      Parameters.Add(new LookupParameter<IntValue>("Generations", "rename value to iterations", "Iterations"));
      Parameters.Add(new LookupParameter<IntValue>("MaximumGenerations", "rename value to maximum iterations", "MaximumIterations"));
      Parameters.Add(new ConstrainedValueParameter<IManipulator>("Manipulator", "The actual maniuplator that should be wrapped.",
        new ItemSet<IManipulator>(ApplicationManager.Manager.GetInstances<IManipulator>())));
      ManipulatorParameter.Value =
        ManipulatorParameter.ValidValues.FirstOrDefault(m => m.GetType() == typeof (MultiMutator));
    }

    public override IDeepCloneable Clone(Cloner cloner) { return new MoveGeneratorAdapter(this, cloner); }
    #endregion

    public override IOperation Apply() {
      var successors = new OperationCollection();
      for (int i = 0; i < SampleSize; i++) {
        var subscope = new Scope(i.ToString());
        foreach (var variable in ExecutionContext.Scope.Variables) {
          subscope.Variables.Add(new Variable(variable.Name, (IItem) variable.Value.Clone()));
        }
        ExecutionContext.Scope.SubScopes.Add(subscope);
        successors.Add(ExecutionContext.CreateChildOperation((IOperator) Manipulator.Clone(), subscope));
      }
      successors.Add(base.Apply());
      return successors;
    }
  }
}
