﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using SharpMap.Data;
using SharpMap.Data.Providers;
using System.Data;
using System.IO;
using System.Linq;

namespace HeuristicLab.BioBoost.Persistence {

  public static class ShapeFileLoader {
    public static GeometryFeatureProvider LoadShapeFile(string filename) {
      var shapefile = new ShapeFile(filename, true, true);
      shapefile.Open();
      var table = new FeatureDataTable {TableName = Path.GetFileNameWithoutExtension(shapefile.Filename)};
      bool addColumns = true;
      foreach (var row in shapefile.GetObjectIDsInView(shapefile.GetExtents()).Select(shapefile.GetFeature)) {
        if (addColumns && row.Table != null) {
          foreach (DataColumn col in row.Table.Columns)
            table.Columns.Add(col.ColumnName, col.DataType);
          addColumns = false;
        }
        var newRow = (FeatureDataRow) table.LoadDataRow(row.ItemArray, LoadOption.OverwriteChanges);
        newRow.Geometry = row.Geometry;
      }
      return new GeometryFeatureProvider(table);
    }
  }
}
