﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;

namespace HeuristicLab.BioBoost.Utils {
  public static class DictionaryExtensions {

    public static void ApplyAt<TKey, TValue>(this IDictionary<TKey, TValue> dict, TKey key, Action<TValue> f) {
      TValue value;
      if (dict.TryGetValue(key, out value)) {
        f(value);
      }
    }

    public static TReturn ApplyAt<TKey, TValue, TReturn>(this IDictionary<TKey, TValue> dict, TKey key, Func<TValue, TReturn> f, Func<TReturn> fnull) {
      TValue value;
      if (dict.TryGetValue(key, out value)) {
        return f(value);
      }
      return fnull();
    }

    public static TValue ElementAtOrDefault<TKey, TValue>(this IDictionary<TKey, TValue> dict, TKey key, TValue defaultValue) {
      TValue value;
      if (dict.TryGetValue(key, out value)) {
        return value;
      } else {
        return defaultValue;
      }
    }
  }
}
